import React from 'react';
import { Auth } from './components/Auth';
import { AuthCallback } from './components/AuthCallback';
import { RefreshApp } from './components/RefreshApp';
import { SecureScreen } from './components/SecureScreen';
import { HeadersBuilder } from './authServices/HeadersUtil';
import { AuthContext, RBAuthInitialToken } from './roles-based-auth/context';
import { TokenUtil } from './authServices/TokenUtilities';
import { RequestBuilder, HTTPMethod } from './authServices/RequestBuilder';
import { ApiAccessBuilder } from './authServices/RequestWrapper';
import { BaseAuthApiWrapper } from './authServices/BaseAuthApiWrapper';
import { Can } from './components/Can';
import { RBAuthErrors } from './rb-auth-errors';
export { AuthContext, Auth, RefreshApp, SecureScreen, AuthCallback, Can, TokenUtil, HeadersBuilder, RequestBuilder, HTTPMethod, ApiAccessBuilder, RBAuthInitialToken, BaseAuthApiWrapper, RBAuthErrors, };
/**
 * TYPES
 */
/**
 * SecureRoute Types
 */
export declare type RBAuthRedirect = React.FC<{
    to: string;
}>;
/**
 * Rule Types
 */
declare type RBAuthStaticRulesType = string[];
declare type RBAuthDynamicRulesType = {
    [key: string]: (...args: any) => boolean;
};
export declare type RBAuthRulesInterface<RoleType extends string> = {
    [key in RoleType]: {
        static?: RBAuthStaticRulesType;
        dynamic?: RBAuthDynamicRulesType;
    };
};
/**
 * User types
 */
declare type RBAuthGenericRoles<T extends string> = 'admin' | 'public' | T;
export interface RBAuthUserModelWithRole<T extends string> {
    role: RBAuthGenericRoles<T>;
}
/**
 * Context types
 */
export declare type KnownAuthProcess<R> = (...args: any) => Promise<R>;
export declare type UnknownAuthProcess = <R>(...args: any) => Promise<R | unknown | void>;
export declare type UnknownRefreshProcess = <R>(reloadFlag?: boolean) => Promise<R | unknown | void>;
export declare type RBAuthContextType<TUser extends RBAuthUserModelWithRole<string> = RBAuthUserModelWithRole<string>, TRules extends RBAuthRulesInterface<string> = RBAuthRulesInterface<RBAuthBaseRoles>, LoginType = UnknownAuthProcess, LogOutType = UnknownAuthProcess, SignUpType = UnknownAuthProcess, SilentAuthType = UnknownAuthProcess, HandleAuthType = UnknownAuthProcess, RefreshTokens = UnknownRefreshProcess, TApis = Record<string, unknown>> = {
    isAuth: boolean;
    reloading: boolean;
    user: TUser;
    logic: {
        login: LoginType;
        signup: SignUpType;
        silent: SilentAuthType;
        handle: HandleAuthType;
        logout: LogOutType;
        refresh: RefreshTokens;
        apis: TApis;
    };
    routes: {
        public: string;
        private: string;
    };
    rules?: TRules;
};
/**
 * Types for Auth processing
 */
export declare type SetterType = (variable: any) => void;
export interface AuthApiInterface {
    login: KnownAuthProcess<{
        tokens: RBAuthTokensType;
        user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
    }>;
    logout: UnknownAuthProcess;
    signup: UnknownAuthProcess;
    handle: KnownAuthProcess<{
        tokens: RBAuthTokensType;
        user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
    }>;
    silent: KnownAuthProcess<{
        tokens: RBAuthTokensType;
        user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
    }>;
}
/**
 * RBAuthStorageType
 * @implNote
 * If you create any 'get Tokens' functionality, please make it private, either by using ES6 classes or
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Closures#Emulating_private_methods_with_closures
 */
export declare type RBAuthStorageType = {
    accessToken: string;
    refreshToken: string;
    openIdToken: string;
    tokenType: string;
    expiresIn: string;
    scope: string;
    setTokens: (tokens: RBAuthTokensType) => void;
};
/**
 * Roles Types
 */
export declare type RBAuthBaseRoles = 'admin' | 'public';
export declare type PartialAuthApi = Partial<AuthApiInterface>;
export declare type RBAuthTokensType = {
    accessToken: string;
    refreshToken: string;
    openIdToken: string;
    expiresIn: string;
    scope: string;
    tokenType: string;
};
/**
 * React Context Type - for your AppAuthContext
 */
export declare type RBAuthReactContext<TUser extends RBAuthUserModelWithRole<string>, TRules extends RBAuthRulesInterface<string>, LoginType extends UnknownAuthProcess = UnknownAuthProcess, LogOutType extends UnknownAuthProcess = UnknownAuthProcess, SignUpType extends UnknownAuthProcess = UnknownAuthProcess, HandleAuthType extends UnknownAuthProcess = UnknownAuthProcess, SilentAuthType extends UnknownAuthProcess = UnknownAuthProcess, RefreshTokensType extends UnknownAuthProcess = UnknownAuthProcess, TApi extends Record<string, any> = Record<string, any>> = React.Context<RBAuthContextType<TUser, TRules, LoginType, LogOutType, SignUpType, SilentAuthType, HandleAuthType, RefreshTokensType, TApi>>;
